import { ISO_8601_FULL, PROPERTY_TYPE_BOOLEAN, PROPERTY_TYPE_DATETIME, PROPERTY_TYPE_NULL, PROPERTY_TYPE_NUMBER, PROPERTY_TYPE_STRING } from '../common/constants.js';
import { isArray, isObject, isString } from '../common/utils.js';
import { DBRecordInstance } from '../sdk';
import { Transaction } from '../sdk/transaction.js';
import { DEFAULT_BASE_PATH, DEFAULT_HOST, DEFAULT_PORT, DEFAULT_PROTOCOL } from './constants.js';
export const buildTransactionHeader = (txId) => txId ?
    {
        ['x-transaction-id']: txId
    }
    : undefined;
export const isTransaction = (input) => isString(input) || input instanceof Transaction;
export const pickTransaction = (input) => (isTransaction(input) ? input : undefined);
export const pickTransactionId = (input) => isTransaction(input) ?
    input instanceof Transaction ?
        input.id
        : input
    : undefined;
export const pickRecordId = (input) => {
    if (isString(input)) {
        return input;
    }
    else if (input instanceof DBRecordInstance && input.data) {
        return input.data.__id;
    }
    else if ('__id' in input) {
        return input.__id;
    }
    return undefined;
};
export const createSearchParams = (labelOrSearchParams, searchParamsOrTransaction) => {
    var _a, _b, _c;
    const isFirstArgString = isString(labelOrSearchParams);
    const isFirstArgUUID = isUUID(labelOrSearchParams);
    const isSecondArgTransaction = isTransaction(searchParamsOrTransaction);
    const isEmptySearchParams = isSecondArgTransaction || !isObject(searchParamsOrTransaction);
    if (isFirstArgString) {
        const baseParams = isFirstArgUUID ?
            { id: labelOrSearchParams }
            : { searchParams: { labels: [labelOrSearchParams] } };
        return isEmptySearchParams ? Object.assign(Object.assign({}, baseParams), { searchParams: Object.assign({}, baseParams.searchParams) }) : Object.assign(Object.assign({}, baseParams), { searchParams: Object.assign(Object.assign({}, searchParamsOrTransaction), { labels: [
                    ...((_b = (_a = baseParams.searchParams) === null || _a === void 0 ? void 0 : _a.labels) !== null && _b !== void 0 ? _b : []),
                    ...((_c = searchParamsOrTransaction.labels) !== null && _c !== void 0 ? _c : [])
                ] }) });
    }
    else {
        return { searchParams: labelOrSearchParams !== null && labelOrSearchParams !== void 0 ? labelOrSearchParams : {} };
    }
};
export const isUUID = (value) => {
    const regex = /^[0-9a-fA-F]{8}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{4}\b-[0-9a-fA-F]{12}$/;
    return regex.test(value);
};
export const arrayIsConsistent = (arr) => arr.every((item) => typeof item === typeof arr[0]);
export const getValueParameters = (value) => {
    if (Array.isArray(value)) {
        return {
            isEmptyArray: value.length === 0,
            isEmptyStringsArray: value.every((v) => v === ''),
            isInconsistentArray: !arrayIsConsistent(value)
        };
    }
    else {
        return { isEmptyString: value === '' };
    }
};
export const suggestPropertyType = (value) => {
    if (typeof value === PROPERTY_TYPE_STRING) {
        return ISO_8601_FULL.test(value) ? PROPERTY_TYPE_DATETIME : PROPERTY_TYPE_STRING;
    }
    else if (typeof value === PROPERTY_TYPE_NUMBER) {
        return PROPERTY_TYPE_NUMBER;
    }
    else if (typeof value === PROPERTY_TYPE_BOOLEAN) {
        return PROPERTY_TYPE_BOOLEAN;
    }
    else if (value === null) {
        return PROPERTY_TYPE_NULL;
    }
    else {
        return PROPERTY_TYPE_STRING;
    }
};
const processArrayValue = (value, suggestTypes) => {
    const { isEmptyArray, isInconsistentArray } = getValueParameters(value);
    if (isEmptyArray) {
        return { type: PROPERTY_TYPE_STRING, value: [] };
    }
    if (isInconsistentArray || !suggestTypes) {
        return { type: PROPERTY_TYPE_STRING, value: value.map(String) };
    }
    return { type: suggestPropertyType(value[0]), value };
};
const processNonArrayValue = (value, suggestTypes) => {
    if (!suggestTypes) {
        return { type: PROPERTY_TYPE_STRING, value: String(value) };
    }
    const type = suggestPropertyType(value);
    return { type, value: type === PROPERTY_TYPE_NULL ? null : value };
};
export const prepareProperties = (data, options = { suggestTypes: true }) => Object.entries(data).map(([name, value]) => {
    const { type, value: processedValue } = isArray(value) ?
        processArrayValue(value, options.suggestTypes)
        : processNonArrayValue(value, options.suggestTypes);
    return { name, type, value: processedValue };
});
export const normalizeRecord = ({ label, options = { suggestTypes: true }, payload }) => ({
    label,
    properties: prepareProperties(payload, options)
});
export const buildUrl = (props) => {
    let protocol = DEFAULT_PROTOCOL;
    let host = DEFAULT_HOST;
    let port = DEFAULT_PORT;
    let basePath = DEFAULT_BASE_PATH;
    if ('url' in props) {
        const url = new URL(props.url || (document === null || document === void 0 ? void 0 : document.URL));
        protocol = url.protocol.replace(':', '');
        host = url.hostname;
        port = parseInt(url.port ||
            (protocol === 'http' ? '80'
                : protocol === 'https' ? '443'
                    : ''));
    }
    if ('host' in props && 'port' in props && 'protocol' in props) {
        protocol = props.protocol.replace(':', '');
        host = props.host;
        port = props.port;
    }
    // Ensure the basePath starts with a '/'
    if (basePath && !basePath.startsWith('/')) {
        basePath = '/' + basePath;
    }
    // If the port is the default for the protocol (80 for http, 443 for https), it can be omitted
    let portString = '';
    if (!((protocol === 'http' && port === 80) || (protocol === 'https' && port === 443))) {
        portString = ':' + port;
    }
    return `${protocol}://${host}${portString}${basePath}`;
};
